from abc import ABC, abstractmethod

from chosco.annotation.domain.aggregate_roots.media_item import MediaItem


class AnnotationTaskProvider(ABC):
    @abstractmethod
    def get_number_of_annotations(self):
        raise NotImplementedError

    @abstractmethod
    def get_number_pending_annotations(self):
        raise NotImplementedError

    @abstractmethod
    def get_total_annotations(self):
        raise NotImplementedError

    @abstractmethod
    def get_status(self):
        raise NotImplementedError

    @abstractmethod
    def next(self) -> MediaItem:
        raise NotImplementedError

    @abstractmethod
    def previous(self) -> MediaItem:
        raise NotImplementedError

    @abstractmethod
    def confirm_current_task(self, task: MediaItem):
        raise NotImplementedError



from chosco.annotation.application.annotator_builder import AnnotatorBuilder
from chosco.annotation.domain.annotation import Annotation
from chosco.annotation.domain.annotation_repository import AnnotationRepository
from chosco.annotation.domain.annotation_session import AnnotationSession
from chosco.annotation.infrastructure.inmemory_annotation_repository import (
    InMemoryAnnotationRepository,
)
from chosco.annotation.infrastructure.json_annotation_task_provider import (
    JsonAnnotationTaskProvider,
)
from chosco.annotation.infrastructure.json_folder_annotation_repository import (
    JsonFolderAnnotationRepository,
)
from chosco.media.repository.infrastructure.hardcoded_web_media_repository import (
    HardcodedWebMediaRepository,
)
from chosco.media.repository.infrastructure.gcp_bucket_media_repository import (
    GoogleCloudMediaRepository,
)
from chosco.media.repository.infrastructure.gcp_bucket_multimedia_repository import (
    GoogleCloudMultimediaRepository,
)
from chosco.annotation.domain.aggregate_roots.media_item import MediaItem
from chosco.media.retriever.domain.media_retriever import MediaRetriever
from chosco.view.interactor.annotation_view_interactor import AnnotationViewInteractor
from chosco.view.interactor.domain.metadata_formatter import MetadataFormatter



from typing import Dict


class MediaItem(object):
    def __init__(self, media_id: str, media_bucket: str = None, metadata: Dict = None):
        self.media_id = media_id
        self.media_bucket = media_bucket
        self.metadata = metadata

    def __eq__(self, other):
        return self.media_id == other.media_id

    def to_dict(self) -> Dict:
        return dict(
            media_id=self.media_id,
            media_bucket=self.media_bucket,
            metadata=self.metadata,
        )

    @staticmethod
    def from_dict(media_item_dict: Dict):
        media_id = media_item_dict.get("media_id")
        media_bucket = media_item_dict.get("media_bucket", None)
        metadata = media_item_dict.get("metadata", None)
        return MediaItem(media_id, media_bucket, metadata)
